@@ -25,6 +25,9 @@
  *
  * #GUPnPContext wraps the networking bits that are used by the various
  * GUPnP classes. It automatically starts a web server on demand.
+ *
+ * For debugging, it is possible to see the messages being sent and received by
+ * exporting #GUPNP_DEBUG.
  */
 
 #include <config.h>
@@ -43,8 +46,7 @@
 #include <net/if.h>
 #include <ifaddrs.h>
 #include <libsoup/soup-address.h>
-
-#include "xdgmime/xdgmime.h"
+#include <glib/gstdio.h>
 
 #include "gupnp-context.h"
 #include "gupnp-context-private.h"
@@ -73,12 +75,18 @@ enum {
         PROP_HOST_IP,
         PROP_PORT,
         PROP_SERVER,
+        PROP_SESSION,
         PROP_SUBSCRIPTION_TIMEOUT
 };
 
 #define LOOPBACK_IP "127.0.0.1"
 
-/**
+typedef struct {
+        char *local_path;
+        char *server_path;
+} HostPathData;
+
+/*
  * Generates the default server ID.
  **/
 static char *
@@ -94,11 +102,12 @@ make_server_id (void)
                                 VERSION);
 }
 
-/**
- * Looks up the IP address of the first non-loopback network interface.
- **/
+/*
+ * Get the host IP for the specified interface, or the first up and non-loopback
+ * interface if no name is specified.
+ */
 static char *
-get_default_host_ip (void)
+get_host_ip (const char *name)
 {
         struct ifaddrs *ifa_list, *ifa;
         char *ret;
@@ -125,6 +134,10 @@ get_default_host_ip (void)
                     !(ifa->ifa_flags & IFF_UP))
                         continue;
 
+                /* If a name was specified, check it */
+                if (name && strcmp (name, ifa->ifa_name) != 0)
+                        continue;
+                
                 p = NULL;
 
                 switch (ifa->ifa_addr->sa_family) {
@@ -158,6 +171,42 @@ get_default_host_ip (void)
         return ret;
 }
 
+/*
+ * Get the host IP of the interface used for the default route.  On any error,
+ * the first up and non-loopback interface is used.
+ */
+static char *
+get_default_host_ip (void)
+{
+        FILE *fp;
+        int ret;
+        char dev[32];
+        unsigned long dest;
+        gboolean found = FALSE;
+        
+        /* TODO: error checking */
+
+        fp = fopen ("/proc/net/route", "r");
+        
+        /* Skip the header */
+        fscanf (fp, "%*[^\n]\n");
+        
+        while ((ret = fscanf (fp,
+                              "%31s %lx %*x %*X %*d %*d %*d %*x %*d %*d %*d",
+                              dev, &dest)) != EOF) {
+                /* If we got a device name and destination, and the destination
+                   is 0, then we have the default route */
+                if (ret == 2 && dest == 0) {
+                        found = TRUE;
+                        break;
+                }
+        }
+        
+        fclose (fp);
+
+        return get_host_ip (found ? dev : NULL);
+}
+
 static void
 gupnp_context_init (GUPnPContext *context)
 {
@@ -168,13 +217,39 @@ gupnp_context_init (GUPnPContext *context)
                                              GUPNP_TYPE_CONTEXT,
                                              GUPnPContextPrivate);
 
-        context->priv->session = soup_session_async_new ();
-
         server_id = make_server_id ();
         gssdp_client_set_server_id (GSSDP_CLIENT (context), server_id);
         g_free (server_id);
 }
 
+static GObject *
+gupnp_context_constructor (GType                  type,
+                           guint                  n_props,
+                           GObjectConstructParam *props)
+{
+	GObject *object;
+	GUPnPContext *context;
+
+	object = G_OBJECT_CLASS (gupnp_context_parent_class)->constructor
+                (type, n_props, props);
+	context = GUPNP_CONTEXT (object);
+
+        context->priv->session = soup_session_async_new_with_options
+                (SOUP_SESSION_IDLE_TIMEOUT,
+                 60,
+                 SOUP_SESSION_ASYNC_CONTEXT,
+                 gssdp_client_get_main_context (GSSDP_CLIENT (context)),
+                 NULL);
+
+	if (g_getenv ("GUPNP_DEBUG")) {
+		SoupLogger *logger;
+		logger = soup_logger_new (SOUP_LOGGER_LOG_BODY, -1);
+		soup_logger_attach (logger, context->priv->session);
+	}
+
+	return object;
+}
+
 static void
 gupnp_context_set_property (GObject      *object,
                             guint         property_id,
@@ -224,6 +299,10 @@ gupnp_context_get_property (GObject    *object,
                 g_value_set_object (value,
                                     gupnp_context_get_server (context));
                 break;
+        case PROP_SESSION:
+                g_value_set_object (value,
+                                    gupnp_context_get_session (context));
+                break;
         case PROP_SUBSCRIPTION_TIMEOUT:
                 g_value_set_uint (value,
                                   gupnp_context_get_subscription_timeout
@@ -282,6 +361,7 @@ gupnp_context_class_init (GUPnPContextClass *klass)
 
         object_class = G_OBJECT_CLASS (klass);
 
+        object_class->constructor  = gupnp_context_constructor;
         object_class->set_property = gupnp_context_set_property;
         object_class->get_property = gupnp_context_get_property;
         object_class->dispose      = gupnp_context_dispose;
@@ -342,6 +422,23 @@ gupnp_context_class_init (GUPnPContextClass *klass)
                                       G_PARAM_STATIC_NICK |
                                       G_PARAM_STATIC_BLURB));
 
+        /**
+         * GUPnPContext:session
+         *
+         * The #SoupSession object used by GUPnP.
+         **/
+        g_object_class_install_property
+                (object_class,
+                 PROP_SESSION,
+                 g_param_spec_object ("session",
+                                      "SoupSession",
+                                      "SoupSession object",
+                                      SOUP_TYPE_SESSION,
+                                      G_PARAM_READABLE |
+                                      G_PARAM_STATIC_NAME |
+                                      G_PARAM_STATIC_NICK |
+                                      G_PARAM_STATIC_BLURB));
+
         /**
          * GUPnPContext:subscription-timeout
          *
@@ -365,18 +462,32 @@ gupnp_context_class_init (GUPnPContextClass *klass)
                                     G_PARAM_STATIC_BLURB));
 }
 
+/**
+ * gupnp_context_get_session
+ * @context: A #GUPnPContext
+ *
+ * Get the #SoupSession object that GUPnP is using.
+ *
+ * Return value: The #SoupSession used by GUPnP. Do not unref this when
+ * finished.
+ **/
 SoupSession *
-_gupnp_context_get_session (GUPnPContext *context)
+gupnp_context_get_session (GUPnPContext *context)
 {
+        g_return_val_if_fail (GUPNP_IS_CONTEXT (context), NULL);
+
         return context->priv->session;
 }
 
-/**
+/*
  * Default server handler: Return 404 not found.
  **/
 static void
-default_server_handler (SoupServerContext *server_context,
-                        SoupMessage       *msg,
+default_server_handler (SoupServer        *server,
+                        SoupMessage       *msg, 
+                        const char        *path,
+                        GHashTable        *query,
+                        SoupClientContext *client,
                         gpointer           user_data)
 {
         soup_message_set_status (msg, SOUP_STATUS_NOT_FOUND);
@@ -386,7 +497,9 @@ default_server_handler (SoupServerContext *server_context,
  * gupnp_context_get_server
  * @context: A #GUPnPContext
  *
- * Return value: The #SoupServer HTTP server used by GUPnP.
+ * Get the #SoupServer HTTP server that GUPnP is using.
+ *
+ * Return value: The #SoupServer used by GUPnP. Do not unref this when finished.
  **/
 SoupServer *
 gupnp_context_get_server (GUPnPContext *context)
@@ -394,12 +507,16 @@ gupnp_context_get_server (GUPnPContext *context)
         g_return_val_if_fail (GUPNP_IS_CONTEXT (context), NULL);
 
         if (context->priv->server == NULL) {
-                context->priv->server = soup_server_new (SOUP_SERVER_PORT,
-                                                         context->priv->port,
-                                                         NULL);
+                context->priv->server = soup_server_new
+                        (SOUP_SERVER_PORT,
+                         context->priv->port,
+                         SOUP_SERVER_ASYNC_CONTEXT,
+                         gssdp_client_get_main_context (GSSDP_CLIENT (context)),
+                         NULL);
 
-                soup_server_add_handler (context->priv->server, NULL, NULL,
-                                         default_server_handler, NULL, context);
+                soup_server_add_handler (context->priv->server, NULL,
+                                         default_server_handler, context,
+                                         NULL);
 
                 soup_server_run_async (context->priv->server);
         }
@@ -407,7 +524,7 @@ gupnp_context_get_server (GUPnPContext *context)
         return context->priv->server;
 }
 
-/**
+/*
  * Makes an URL that refers to our server.
  **/
 static char *
@@ -438,13 +555,15 @@ _gupnp_context_get_server_url (GUPnPContext *context)
 
 /**
  * gupnp_context_new
- * @main_context: A #GMainContext, or NULL to use the default one
- * @host_ip: The local host's IP address, or NULL to use the IP address
+ * @main_context: A #GMainContext, or %NULL to use the default one
+ * @host_ip: The local host's IP address, or %NULL to use the IP address
  * of the first non-loopback network interface.
  * @port: Port to run on, or 0 if you don't care what port is used.
- * @error: A location to store a #GError, or NULL
+ * @error: A location to store a #GError, or %NULL
  *
- * Return value: A new #GUPnPContext object.
+ * Create a new #GUPnPContext with the specified @main_context, @host_ip and @port.
+ *
+ * Return value: A new #GUPnPContext object, or %NULL on an error
  **/
 GUPnPContext *
 gupnp_context_new (GMainContext *main_context,
@@ -464,7 +583,9 @@ gupnp_context_new (GMainContext *main_context,
  * gupnp_context_get_host_ip
  * @context: A #GUPnPContext
  *
- * Return value: The IP address we advertise ourselves as using. Do not free.
+ * Get the IP address we advertise ourselves as using.
+ *
+ * Return value: The IP address. This string should not be freed.
  **/
 const char *
 gupnp_context_get_host_ip (GUPnPContext *context)
@@ -481,6 +602,8 @@ gupnp_context_get_host_ip (GUPnPContext *context)
  * gupnp_context_get_port
  * @context: A #GUPnPContext
  *
+ * Get the port that the SOAP server is running on.
+ 
  * Return value: The port the SOAP server is running on.
  **/
 guint
@@ -518,8 +641,10 @@ gupnp_context_set_subscription_timeout (GUPnPContext *context,
  * gupnp_context_get_subscription_timeout
  * @context: A #GUPnPContext
  *
- * Return value: The event subscription timeout in seconds, or 0, meaning
- * no timeout.
+ * Get the event subscription timeout (in seconds), or 0 meaning there is no
+ * timeout.
+ * 
+ * Return value: The event subscription timeout in seconds.
  **/
 guint
 gupnp_context_get_subscription_timeout (GUPnPContext *context)
@@ -529,163 +654,292 @@ gupnp_context_get_subscription_timeout (GUPnPContext *context)
         return context->priv->subscription_timeout;
 }
 
-/* Modified from libsoup simple-httpd.c */
-static void
-hosting_server_handler (SoupServerContext *server_context,
-                        SoupMessage       *msg,
-                        gpointer           user_data)
+/* Construct a local path from @requested path, removing the last slash
+ * if any to make sure we append the locale suffix in a canonical way. */
+static char *
+construct_local_path (const char   *requested_path,
+                      HostPathData *host_path_data)
 {
-        const char *local_path, *lang, *mime;
-        char *path, *path_to_open, *path_locale, *slash;
-        SoupMethodId method;
+        GString *str;
+        int len;
+
+        if (!requested_path || *requested_path == 0)
+                return g_strdup (host_path_data->local_path);
+
+        if (*requested_path != '/')
+                return NULL; /* Absolute paths only */
+
+        str = g_string_new (host_path_data->local_path);
+
+        /* Skip the length of the path relative to which @requested_path
+         * is specified. */
+        requested_path += strlen (host_path_data->server_path);
+
+        /* Strip the last slashes to make sure we append the locale suffix
+         * in a canonical way. */
+        len = strlen (requested_path);
+        while (requested_path[len - 1] == '/')
+                len--;
+
+        g_string_append_len (str,
+                             requested_path,
+                             len);
+
+        return g_string_free (str, FALSE);
+}
+
+/* Append locale suffix to @local_path. */
+static char *
+append_locale (const char *local_path, GList *locales)
+{
+        if (!locales)
+                return g_strdup (local_path);
+
+        return g_strdup_printf ("%s.%s",
+                                local_path,
+                                (char *) locales->data);
+}
+
+/* Redirect @msg to the same URI, but with a slash appended. */
+static void
+redirect_to_folder (SoupMessage *msg)
+{
+        char *uri, *redir_uri;
+
+        uri = soup_uri_to_string (soup_message_get_uri (msg),
+                                  FALSE);
+        redir_uri = g_strdup_printf ("%s/", uri);
+        soup_message_headers_append (msg->response_headers,
+                                     "Location", redir_uri);
+        soup_message_set_status (msg,
+                                 SOUP_STATUS_MOVED_PERMANENTLY);
+        g_free (redir_uri);
+        g_free (uri);
+}
+
+/* Serve @path. Note that we do not need to check for path including bogus
+ * '..' as libsoup does this for us. */
+static void
+host_path_handler (SoupServer        *server,
+                   SoupMessage       *msg, 
+                   const char        *path,
+                   GHashTable        *query,
+                   SoupClientContext *client,
+                   gpointer           user_data)
+{
+        char *local_path, *path_to_open;
         struct stat st;
-        int fd, path_offset;
-        GList *locales;
+        int status;
+        GList *locales, *orig_locales;
+        GMappedFile *mapped_file;
+        GError *error;
 
-        local_path = (const char *) user_data;
+        orig_locales = NULL;
+        locales      = NULL;
+        local_path   = NULL;
+        path_to_open = NULL;
 
-        path = soup_uri_to_string (soup_message_get_uri (msg), TRUE);
-
-        method = soup_method_get_id (msg->method);
-        if (method != SOUP_METHOD_ID_GET && method != SOUP_METHOD_ID_HEAD) {
+        if (msg->method != SOUP_METHOD_GET &&
+            msg->method != SOUP_METHOD_HEAD) {
                 soup_message_set_status (msg, SOUP_STATUS_NOT_IMPLEMENTED);
-                locales = NULL;
+
+                goto DONE;
+        }
+
+        /* Construct base local path */
+        local_path = construct_local_path (path, (HostPathData *) user_data);
+        if (!local_path) {
+                soup_message_set_status (msg, SOUP_STATUS_BAD_REQUEST);
+
                 goto DONE;
         }
 
         /* Get preferred locales */
-        locales = accept_language_get_locales (msg);
-
-        if (path) {
-                if (*path != '/') {
-                        soup_message_set_status (msg, SOUP_STATUS_BAD_REQUEST);
-                        goto DONE;
-                }
-
-                /* Skip the server path */
-                path_offset = strlen (server_context->handler->path);
-        } else {
-                path = g_strdup ("");
-
-                path_offset = 0;
-        }
-
-        path_locale = NULL;
-
- NEXT_LOCALE:
-        g_free (path_locale);
-
-        if (locales) {
-                path_locale = g_strdup_printf ("%s.%s",
-                                               path + path_offset,
-                                               (char *) locales->data);
-        } else
-                path_locale = g_strdup (path + path_offset);
-
-        path_to_open = g_build_filename (local_path, path_locale, NULL);
+        orig_locales = locales = http_request_get_accept_locales (msg);
 
  AGAIN:
-        if (stat (path_to_open, &st) == -1) {
-                g_free (path_to_open);
+        /* Add locale suffix if available */
+        path_to_open = append_locale (local_path, locales);
+
+        /* See what we've got */
+        if (g_stat (path_to_open, &st) == -1) {
                 if (errno == EPERM)
                         soup_message_set_status (msg, SOUP_STATUS_FORBIDDEN);
                 else if (errno == ENOENT) {
                         if (locales) {
+                                g_free (path_to_open);
+
                                 locales = locales->next;
-                                goto NEXT_LOCALE;
+
+                                goto AGAIN;
                         } else
                                 soup_message_set_status (msg,
                                                          SOUP_STATUS_NOT_FOUND);
                 } else
                         soup_message_set_status
                                 (msg, SOUP_STATUS_INTERNAL_SERVER_ERROR);
+
                 goto DONE;
         }
 
         /* Handle directories */
         if (S_ISDIR (st.st_mode)) {
-                slash = strrchr (path_to_open, '/');
-                if (!slash || slash[1]) {
-                        char *uri, *redir_uri;
+                if (!g_str_has_suffix (path, "/")) {
+                        redirect_to_folder (msg);
 
-                        uri = soup_uri_to_string (soup_message_get_uri (msg),
-                                                  FALSE);
-                        redir_uri = g_strdup_printf ("%s/", uri);
-                        soup_message_add_header (msg->response_headers,
-                                                 "Location", redir_uri);
-                        soup_message_set_status (msg,
-                                                 SOUP_STATUS_MOVED_PERMANENTLY);
-                        g_free (redir_uri);
-                        g_free (uri);
-                        g_free (path_to_open);
                         goto DONE;
                 }
 
+                /* This incorporates the locale portion in the folder name
+                 * intentionally. */
+                g_free (local_path);
+                local_path = g_build_filename (path_to_open,
+                                               "index.html",
+                                               NULL);
+
                 g_free (path_to_open);
-                path_to_open = g_build_filename (local_path,
-                                                 path_locale,
-                                                 "index.html", NULL);
+
                 goto AGAIN;
         }
 
-        g_free (path_locale);
+        /* Map file */
+        error = NULL;
+        mapped_file = g_mapped_file_new (path_to_open, FALSE, &error);
+
+        if (mapped_file == NULL) {
+                g_warning ("Unable to map file %s: %s",
+                           path_to_open, error->message);
+
+                g_error_free (error);
 
-        /* Read file */
-        fd = open (path_to_open, O_RDONLY);
-        g_free (path_to_open);
-        if (fd == -1) {
                 soup_message_set_status (msg,
                                          SOUP_STATUS_INTERNAL_SERVER_ERROR);
+
                 goto DONE;
         }
 
-        msg->response.owner = SOUP_BUFFER_SYSTEM_OWNED;
-        msg->response.length = st.st_size;
+        /* Handle method (GET or HEAD) */
+        status = SOUP_STATUS_OK;
 
-        msg->response.body = g_malloc (msg->response.length);
-        read (fd, msg->response.body, msg->response.length);
+        if (msg->method == SOUP_METHOD_GET) {
+                gsize offset, length;
+                gboolean have_range;
+                SoupBuffer *buffer;
 
-        close (fd);
+                /* Find out range */
+                have_range = FALSE;
+
+                offset = 0;
+                length = st.st_size;
+
+                if (!http_request_get_range (msg,
+                                             &have_range,
+                                             &offset,
+                                             &length)) {
+                        soup_message_set_status
+                                (msg,
+                                 SOUP_STATUS_REQUESTED_RANGE_NOT_SATISFIABLE);
+
+                        goto DONE;
+                }
+
+                if (have_range && (length < 0                   ||
+                                   offset < 0                   ||
+                                   length > st.st_size - offset ||
+                                   offset >= st.st_size)) {
+                        soup_message_set_status
+                                (msg,
+                                 SOUP_STATUS_REQUESTED_RANGE_NOT_SATISFIABLE);
+
+                        goto DONE;
+                }
+
+                /* Add requested content */
+                buffer = soup_buffer_new_with_owner
+                             (g_mapped_file_get_contents (mapped_file) + offset,
+                              length,
+                              mapped_file,
+                              (GDestroyNotify) g_mapped_file_free);
+
+                soup_message_body_append_buffer (msg->response_body, buffer);
+
+                soup_buffer_free (buffer);
+
+                /* Set status */
+                if (have_range) {
+                        http_response_set_content_range (msg,
+                                                         offset,
+                                                         offset + length,
+                                                         st.st_size);
+
+                        status = SOUP_STATUS_PARTIAL_CONTENT;
+                }
+
+        } else if (msg->method == SOUP_METHOD_HEAD) {
+                char *length;
+
+                length = g_strdup_printf ("%lu", (gulong) st.st_size);
+	        soup_message_headers_append (msg->response_headers,
+                                             "Content-Length",
+                                             length);
+                g_free (length);
+
+        } else {
+                g_assert_not_reached ();
+
+        }
 
         /* Set Content-Type */
-        mime = xdg_mime_get_mime_type_for_data (msg->response.body,
-                                                msg->response.length);
-
-        soup_message_add_header (msg->response_headers,
-                                 "Content-Type",
-                                 mime);
+        http_response_set_content_type (msg,
+                                        path_to_open, 
+                                        (guchar *) g_mapped_file_get_contents
+                                                                (mapped_file),
+                                        st.st_size);
 
         /* Set Content-Language */
-        if (locales) {
-                http_language_from_locale (locales->data);
+        if (locales)
+               http_response_set_content_locale (msg, locales->data);
 
-                lang = locales->data;
-        } else
-                lang = "en";
+        /* Set Accept-Ranges */
+        soup_message_headers_append (msg->response_headers,
+                                     "Accept-Ranges",
+                                     "bytes");
 
-        soup_message_add_header (msg->response_headers,
-                                 "Content-Language",
-                                 lang);
-
-        /* OK */
-        soup_message_set_status (msg, SOUP_STATUS_OK);
+        /* Set status */
+        soup_message_set_status (msg, status);
 
  DONE:
         /* Cleanup */
-        g_free (path);
+        g_free (path_to_open);
+        g_free (local_path);
 
-        while (locales) {
-                g_free (locales->data);
-                locales = g_list_delete_link (locales, locales);
+        while (orig_locales) {
+                g_free (orig_locales->data);
+                orig_locales = g_list_delete_link (orig_locales, orig_locales);
         }
 }
 
-static void
-unregister_hosting_server_handler (SoupServer        *server,
-                                   SoupServerHandler *handler,
-                                   gpointer           user_data)
+HostPathData *
+host_path_data_new (const char *local_path,
+                    const char *server_path)
 {
-        g_free (user_data);
+        HostPathData *path_data;
+
+        path_data = g_slice_new (HostPathData);
+
+        path_data->local_path  = g_strdup (local_path);
+        path_data->server_path = g_strdup (server_path);
+
+        return path_data;
+}
+
+void
+host_path_data_free (HostPathData *path_data)
+{
+        g_free (path_data->local_path);
+        g_free (path_data->server_path);
+
+        g_slice_free (HostPathData, path_data);
 }
 
 /**
@@ -704,6 +958,7 @@ gupnp_context_host_path (GUPnPContext *context,
                          const char   *server_path)
 {
         SoupServer *server;
+        HostPathData *path_data;
 
         g_return_if_fail (GUPNP_IS_CONTEXT (context));
         g_return_if_fail (local_path != NULL);
@@ -711,10 +966,13 @@ gupnp_context_host_path (GUPnPContext *context,
 
         server = gupnp_context_get_server (context);
 
-        soup_server_add_handler (server, server_path, NULL,
-                                 hosting_server_handler,
-                                 unregister_hosting_server_handler,
-                                 g_strdup (local_path));
+        path_data = host_path_data_new (local_path, server_path);
+
+        soup_server_add_handler (server,
+                                 server_path,
+                                 host_path_handler,
+                                 path_data,
+                                 (GDestroyNotify) host_path_data_free);
 }
 
 /**
