@@ -49,7 +49,7 @@ struct _GUPnPServiceInfoPrivate {
         char *udn;
         char *service_type;
 
-        SoupUri *url_base;
+        SoupURI *url_base;
 
         XmlDocWrapper *doc;
 
@@ -186,16 +186,16 @@ gupnp_service_info_dispose (GObject *object)
         if (info->priv->context) {
                 SoupSession *session;
 
-                session = _gupnp_context_get_session (info->priv->context);
+                session = gupnp_context_get_session (info->priv->context);
 
                 while (info->priv->pending_gets) {
                         GetSCPDURLData *data;
 
                         data = info->priv->pending_gets->data;
 
-                        soup_message_set_status (data->message,
-                                                 SOUP_STATUS_CANCELLED);
-                        soup_session_cancel_message (session, data->message);
+                        soup_session_cancel_message (session,
+                                                     data->message,
+                                                     SOUP_STATUS_CANCELLED);
 
                         get_scpd_url_data_free (data);
 
@@ -319,7 +319,7 @@ gupnp_service_info_class_init (GUPnPServiceInfoClass *klass)
         /**
          * GUPnPServiceInfo:url-base
          *
-         * The URL base (#SoupUri).
+         * The URL base (#SoupURI).
          **/
         g_object_class_install_property
                 (object_class,
@@ -379,7 +379,9 @@ gupnp_service_info_class_init (GUPnPServiceInfoClass *klass)
  * gupnp_service_info_get_context
  * @info: A #GUPnPServiceInfo
  *
- * Return value: The #GUPnPContext associated with @info.
+ * Get the #GUPnPContext associated with @info.
+ *
+ * Returns: A #GUPnPContext.
  **/
 GUPnPContext *
 gupnp_service_info_get_context (GUPnPServiceInfo *info)
@@ -393,7 +395,9 @@ gupnp_service_info_get_context (GUPnPServiceInfo *info)
  * gupnp_service_info_get_location
  * @info: A #GUPnPServiceInfo
  *
- * Return value: The location of the device description file.
+ * Get the location of the device description file.
+ *
+ * Returns: A constant string.
  **/
 const char *
 gupnp_service_info_get_location (GUPnPServiceInfo *info)
@@ -407,9 +411,11 @@ gupnp_service_info_get_location (GUPnPServiceInfo *info)
  * gupnp_service_info_get_url_base
  * @info: A #GUPnPServiceInfo
  *
- * Return value: The URL base.
+ * Get the URL base of this service.
+ *
+ * Returns: A constant #SoupURI.
  **/
-const SoupUri *
+const SoupURI *
 gupnp_service_info_get_url_base (GUPnPServiceInfo *info)
 {
         g_return_val_if_fail (GUPNP_IS_SERVICE_INFO (info), NULL);
@@ -421,7 +427,9 @@ gupnp_service_info_get_url_base (GUPnPServiceInfo *info)
  * gupnp_service_info_get_udn
  * @info: A #GUPnPServiceInfo
  *
- * Return value: The UDN of the containing device.
+ * Get the Unique Device Name of the containing device.
+ *
+ * Returns: A constant string.
  **/
 const char *
 gupnp_service_info_get_udn (GUPnPServiceInfo *info)
@@ -435,7 +443,9 @@ gupnp_service_info_get_udn (GUPnPServiceInfo *info)
  * gupnp_service_info_get_service_type
  * @info: A #GUPnPServiceInfo
  *
- * Return value: The UPnP service type, or NULL.
+ * Get the UPnP service type, or %NULL.
+ *
+ * Returns: A constant string.
  **/
 const char *
 gupnp_service_info_get_service_type (GUPnPServiceInfo *info)
@@ -455,7 +465,9 @@ gupnp_service_info_get_service_type (GUPnPServiceInfo *info)
  * gupnp_service_info_get_id
  * @info: A #GUPnPServiceInfo
  *
- * Return value: The ID, or NULL. g_free() after use.
+ * Get the ID of this service, or %NULL if there is no ID.
+ *
+ * Return value: A string. This string should be freed with g_free() after use.
  **/
 char *
 gupnp_service_info_get_id (GUPnPServiceInfo *info)
@@ -470,7 +482,9 @@ gupnp_service_info_get_id (GUPnPServiceInfo *info)
  * gupnp_service_info_get_scpd_url
  * @info: A #GUPnPServiceInfo
  *
- * Return value: The SCPD URL, or NULL. g_free() after use.
+ * Get the SCPD URL for this service, or %NULL if there is no SCPD.
+ *
+ * Return value: A string. This string should be freed with g_free() after use.
  **/
 char *
 gupnp_service_info_get_scpd_url (GUPnPServiceInfo *info)
@@ -486,7 +500,9 @@ gupnp_service_info_get_scpd_url (GUPnPServiceInfo *info)
  * gupnp_service_info_get_control_url
  * @info: A #GUPnPServiceInfo
  *
- * Return value: The control URL, or NULL. g_free() after use.
+ * Get the control URL for this service, or %NULL..
+ *
+ * Return value: A string. This string should be freed with g_free() after use.
  **/
 char *
 gupnp_service_info_get_control_url (GUPnPServiceInfo *info)
@@ -502,7 +518,9 @@ gupnp_service_info_get_control_url (GUPnPServiceInfo *info)
  * gupnp_service_info_get_event_subscription_url
  * @info: A #GUPnPServiceInfo
  *
- * Return value: The event subscription URL, or NULL. g_free() after use.
+ * Get the event subscription URL for this service, or %NULL.
+ *
+ * Return value: A string. This string should be freed with g_free() after use.
  **/
 char *
 gupnp_service_info_get_event_subscription_url (GUPnPServiceInfo *info)
@@ -517,13 +535,13 @@ gupnp_service_info_get_event_subscription_url (GUPnPServiceInfo *info)
 /**
  * gupnp_service_info_get_introspection
  * @info: A #GUPnPServiceInfo
- * @error: return location for a GError, or NULL
+ * @error: return location for a #GError, or %NULL
  *
  * Note that introspection object is created from the information in service
  * description document (SCPD) provided by the service so it can not be created
  * if the service does not provide an SCPD.
  *
- * Return value: A new #GUPnPServiceIntrospection for this service or NULL.
+ * Return value: A new #GUPnPServiceIntrospection for this service or %NULL.
  * Unref after use.
  **/
 GUPnPServiceIntrospection *
@@ -559,10 +577,10 @@ gupnp_service_info_get_introspection (GUPnPServiceInfo *info,
                 return NULL;
         }
 
-        message_set_user_agent (msg);
+        http_request_set_user_agent (msg);
 
         /* Send off the message */
-        session = _gupnp_context_get_session (info->priv->context);
+        session = gupnp_context_get_session (info->priv->context);
 
         status = soup_session_send_message (session, msg);
         if (!SOUP_STATUS_IS_SUCCESSFUL (status)) {
@@ -573,7 +591,8 @@ gupnp_service_info_get_introspection (GUPnPServiceInfo *info,
                 return NULL;
         }
 
-        scpd = xmlParseMemory (msg->response.body, msg->response.length);
+        scpd = xmlParseMemory (msg->response_body->data,
+                               msg->response_body->length);
 
         g_object_unref (msg);
 
@@ -593,11 +612,12 @@ gupnp_service_info_get_introspection (GUPnPServiceInfo *info,
         return introspection;
 }
 
-/**
+/*
  * SCPD URL downloaded.
- **/
+ */
 static void
-got_scpd_url (SoupMessage    *msg,
+got_scpd_url (SoupSession    *session,
+              SoupMessage    *msg,
               GetSCPDURLData *data)
 {
         GUPnPServiceIntrospection *introspection;
@@ -612,8 +632,8 @@ got_scpd_url (SoupMessage    *msg,
         if (SOUP_STATUS_IS_SUCCESSFUL (msg->status_code)) {
                 xmlDoc *scpd;
 
-                scpd = xmlParseMemory (msg->response.body,
-                                       msg->response.length);
+                scpd = xmlParseMemory (msg->response_body->data,
+                                       msg->response_body->length);
                 if (scpd) {
                         introspection = gupnp_service_introspection_new (scpd);
 
@@ -694,7 +714,7 @@ gupnp_service_info_get_introspection_async
                 return;
         }
 
-        message_set_user_agent (data->message);
+        http_request_set_user_agent (data->message);
 
         data->info      = info;
         data->callback  = callback;
@@ -705,10 +725,10 @@ gupnp_service_info_get_introspection_async
                 g_list_prepend (info->priv->pending_gets,
                                 data);
 
-        session = _gupnp_context_get_session (info->priv->context);
+        session = gupnp_context_get_session (info->priv->context);
 
         soup_session_queue_message (session,
                                     data->message,
-                                    (SoupMessageCallbackFn) got_scpd_url,
+                                    (SoupSessionCallback) got_scpd_url,
                                     data);
 }
